﻿using System;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

namespace AsyncReduxUtils
{
    public static class StopwatchHelper
    {
        static StopwatchHelper()
        {
            Stopwatches = new ConcurrentDictionary<string, TimerInfo>();
        }

        public static ConcurrentDictionary<string, TimerInfo> Stopwatches { get; set; }

        public static string StartNew(TimerType tt)
        {
            var message = GetStartTimerMessage(tt);

            // get token
            var token = Guid.NewGuid().ToString();

            // build TimerInfo
            var sw = Stopwatch.StartNew();
            var ti = new TimerInfo { Type = tt, Stopwatch = sw };

            // store stopwatch with token as key
            Stopwatches.TryAdd(token, ti);

            Debug.WriteLine(message);

            return token;
        }

        public static void Stop(string token)
        { 
            // lookup token and retrieved timer type and stopwatch
            TimerInfo ti;
            Stopwatches.TryGetValue(token, out ti);

            // stop the stopwatch
            ti.Stopwatch.Stop();

            // generate message for output results
            var message = GetStopTimerMessage(ti);

            Debug.WriteLine(message);
        }

        private static string GetStartTimerMessage(TimerType tt)
        {
            var outputMessage = "";

            switch (tt)
            {
                case TimerType.MAIN_TIMER:
                    outputMessage = "Identify Main Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.START_CLICK_TIMER:
                    outputMessage = "Start Click Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.WORKER_TIMER:
                    outputMessage = "Worker Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.UPDATE_RESULT_TIMER:
                    outputMessage = "Update Result Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.CANCEL_CLICK_TIMER:
                    outputMessage = "Cancel Click Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.WAIT_TIMER:
                    outputMessage = "Start Click Thread (Start Waiting): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.CALLBACK_TIMER:
                    outputMessage = "Callback Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.ASYNCOP_POST_TIMER:
                    outputMessage = "AsyncOp Post Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.ASYNC_ENDINVOKE_TIMER:
                    outputMessage = "Async EndInvoke Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;

                case TimerType.ASYNC_PROCESS_TIMER:
                    outputMessage = "Async Process Thread (Started): " + Thread.CurrentThread.ManagedThreadId.ToString();
                    break;
            }

            return outputMessage;
        }

        private static string GetStopTimerMessage(TimerInfo ti)
        {
            var outputMessage = "";

            switch (ti.Type)
            {
                case TimerType.MAIN_TIMER:
                    outputMessage = String.Format("Identify Main Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.START_CLICK_TIMER:
                    outputMessage = String.Format("Start Click Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.WORKER_TIMER:
                    outputMessage = String.Format("Worker Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.UPDATE_RESULT_TIMER:
                    outputMessage = String.Format("Update Result Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.CANCEL_CLICK_TIMER:
                    outputMessage = String.Format("Cancel Click Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.WAIT_TIMER:
                    outputMessage = String.Format("Start Click Thread (End Waiting): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.CALLBACK_TIMER:
                    outputMessage = String.Format("Callback Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.ASYNCOP_POST_TIMER:
                    outputMessage = String.Format("AsyncOp Post Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.ASYNC_ENDINVOKE_TIMER:
                    outputMessage = String.Format("Async EndInvoke Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

                case TimerType.ASYNC_PROCESS_TIMER:
                    outputMessage = String.Format("Async Process Thread (Ending): {0} Elapsed: {1}", Thread.CurrentThread.ManagedThreadId, ti.Stopwatch.ElapsedMilliseconds.ToString());
                    break;

            }

            return outputMessage;
        }
    }

    public enum TimerType
    {
        MAIN_TIMER,
        START_CLICK_TIMER,
        WORKER_TIMER,
        CANCEL_CLICK_TIMER,
        UPDATE_RESULT_TIMER,
        WAIT_TIMER,
        CALLBACK_TIMER,
        ASYNCOP_POST_TIMER,
        ASYNC_ENDINVOKE_TIMER,
        ASYNC_PROCESS_TIMER
    }

    public class TimerInfo
    {
        public TimerType Type {get;set;}
        public Stopwatch Stopwatch {get;set;}
    }
}
