﻿using System;
using System.Diagnostics;
using System.Runtime.Remoting.Messaging;
using System.Threading;
using System.Windows;
using AsyncReduxUtils;

namespace DelegateBasedAsync
{
    public delegate string SlowProcessDelegate();

    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            var token = StopwatchHelper.StartNew(TimerType.MAIN_TIMER);
            StopwatchHelper.Stop(token);

            InitializeComponent();
        }

        private void StartSlowProcessButton_Click(object sender, RoutedEventArgs e)
        {
            var clickToken = StopwatchHelper.StartNew(TimerType.START_CLICK_TIMER);

            // Create new UserControl and add it to the StackPanel
            var uc = new SlowProcessView();
            uc.ViewModel = new SlowProcessViewModel();
            uc.DataContext = uc.ViewModel;
            MainStackPanel.Children.Add(uc);

            SlowProcessDelegate spd = uc.ViewModel.SlowProcess;
            var asyncRes = (AsyncResult)spd.BeginInvoke(EndSlowProcessCallbackMethod, uc);

            var waitToken = StopwatchHelper.StartNew(TimerType.WAIT_TIMER);

            // This thread waits for another thread to complete its work - 
            //     giving it a maximum of 1 second to complete 
            asyncRes.AsyncWaitHandle.WaitOne(TimeSpan.FromSeconds(1));
            StopwatchHelper.Stop(waitToken);

            StopwatchHelper.Stop(clickToken);
        }

        private void EndSlowProcessCallbackMethod(IAsyncResult ar)
        {
            var token = StopwatchHelper.StartNew(TimerType.CALLBACK_TIMER);

            AsyncResult asyncRes = (AsyncResult)ar;
            SlowProcessDelegate spd = (SlowProcessDelegate)asyncRes.AsyncDelegate;
            var uc = (SlowProcessView)asyncRes.AsyncState;

            // Deterministically cleanup the WaitHandle for anyone referencing it
            //     and retrieve the result data produced in spd.BeginInvoke 
            string data = spd.EndInvoke(asyncRes);

            uc.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal,
                new Action<SlowProcessView, string>(UpdateResultText),
                uc,
                data);

            StopwatchHelper.Stop(token);
        }

        private void UpdateResultText(SlowProcessView uc, string data)
        {
            var token = StopwatchHelper.StartNew(TimerType.UPDATE_RESULT_TIMER);

            uc.ResultText.Text = data;

            StopwatchHelper.Stop(token);
        }
    }
}
