﻿using System;
using System.Diagnostics;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Threading;
using AsyncReduxUtils;

namespace DirectThreadAsync
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            var token = StopwatchHelper.StartNew(TimerType.MAIN_TIMER);
            StopwatchHelper.Stop(token);

            InitializeComponent();
        }

        private void StartSlowProcessButton_Click(
            object sender,
            RoutedEventArgs e)
        {
            var token = StopwatchHelper.StartNew(TimerType.START_CLICK_TIMER);

            // Create a new UserControl with a ViewModel
            //   and add it to the StackPanel
            var uc = new SlowProcessView();
            uc.ViewModel = new SlowProcessViewModel();
            uc.DataContext = uc.ViewModel;
            MainStackPanel.Children.Add(uc);

            // Put work on a ThreadPool thread (returns immediately
            //   with boolean)
            bool isQueued = ThreadPool.QueueUserWorkItem(
                SlowProcessWorkItem,
                uc);

            StopwatchHelper.Stop(token);
        }

        private void SlowProcessWorkItem(object o)
        {
            var uc = o as SlowProcessView;

            string data = uc.ViewModel.SlowProcess();

            // Use this object's Dispatcher object to schedule
            //   the UI update back on the UI thread

            // Use separate method
            uc.Dispatcher.BeginInvoke(DispatcherPriority.Normal,
                new Action<SlowProcessView, string>(UpdateResultText),
                uc,
                data);
        }

        private void UpdateResultText(SlowProcessView uc, string data)
        {
            var token = StopwatchHelper.StartNew(TimerType.UPDATE_RESULT_TIMER);

            uc.ResultText.Text = data;

            StopwatchHelper.Stop(token);
        }
    }
}
