﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using AsyncReduxUtils;

namespace TaskBasedAsync
{
    public class SlowProcessViewModel : INotifyPropertyChanged
    {
        public async Task<string> SlowProcessAsync()
        {
            // Synchronous code

            var token = StopwatchHelper.StartNew(TimerType.ASYNC_PROCESS_TIMER);

            // Get a CancellationToken
            var tokenSource = new CancellationTokenSource();
            var cancelToken = tokenSource.Token;
            CancellationTokenSource = tokenSource;

            string resultPromise = await Task.Run<string>(() => SlowProcess(cancelToken), cancelToken);

            // Continuation code

            // Continuation code runs on a thread from the synchronization context captured by await

            ResultData = resultPromise;

            StopwatchHelper.Stop(token);

            return resultPromise;
        }

        public string SlowProcess(CancellationToken cancelToken)
        {
            var swToken = StopwatchHelper.StartNew(TimerType.WORKER_TIMER);

            StartData = "Slow process started.";

            var rv = String.Empty;
            for (var i = 0; i <= 5; i++)
            {
                if (cancelToken.IsCancellationRequested)
                {
                    rv = "Slow process canceled.";
                    break;
                }

                Thread.Sleep(TimeSpan.FromSeconds(1));

                // XAML framework (here, WPF) helps us by marshalling UI update
                //    resulting from data-bound variable update to the Main / UI thread
                ProgressData = i;
            }

            if (rv == String.Empty)
                rv = "Slow process completed.";

            StopwatchHelper.Stop(swToken);

            return rv;
        }

        private string _startData = "";
        public string StartData
        {
            get
            {
                return _startData;
            }
            set
            {
                _startData = value;
                RaisePropertyChanged("StartData");
            }
        }

        private int _progressData = 0;
        public int ProgressData
        {
            get
            {
                return _progressData;
            }
            set
            {
                _progressData = value;
                RaisePropertyChanged("ProgressData");
            }
        }

        private string _resultData = "";
        public string ResultData
        {
            get
            {
                return _resultData;
            }
            set
            {
                _resultData = value;
                RaisePropertyChanged("ResultData");
            }
        }

        public CancellationTokenSource CancellationTokenSource { get; set; }

        #region Implement INotifyPropertyChanged

        public void RaisePropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(name));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

    }
}